from pathlib import Path
import shutil
import sys
import re

CYRILLIC_SYMBOLS = 'абвгдеёжзийклмнопрстуфхцчшщъыьэюяєіїґ'
TRANSLATION = ("a", "b", "v", "g", "d", "e", "e", "j", "z", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u",
               "f", "h", "ts", "ch", "sh", "sch", "", "y", "", "e", "yu", "u", "ja", "je", "ji", "g")
TRANS = {}
for c, l in zip(CYRILLIC_SYMBOLS, TRANSLATION):
    TRANS[ord(c)] = l
    TRANS[ord(c.upper())] = l.upper()


def normalize(name: str) -> str:
    t_name = name.translate(TRANS)
    t_name = re.sub(r'\W', '.', t_name)
    return t_name


JPEG_IMAGES = []
JPG_IMAGES = []
PNG_IMAGES = []
SVG_IMAGES = []

MP3_AUDIO = []
OGG_AUDIO = []
WAV_AUDIO = []
AMR_AUDIO = []

AVI_VIDEO = []
MP4_VIDEO = []
MOV_VIDEO = []
MKV_VIDEO = []

DOC_DOCUMENTS = []
DOCX_DOCUMENTS = []
TXT_DOCUMENTS = []
PDF_DOCUMENTS = []
XLSX_DOCUMENTS = []
PPTX_DOCUMENTS = []

ZIP_ARCHIVES = []
GZ_ARCHIVES = []
TAR_ARCHIVES = []
RAR_ARCHIVES = []

MY_OTHER = []



REGISTER_EXTENSION = {
    'JPEG': JPEG_IMAGES,
    'JPG': JPG_IMAGES,
    'PNG': PNG_IMAGES,
    'SVG': SVG_IMAGES,
    'MP3': MP3_AUDIO,
    'OGG': OGG_AUDIO,
    'WAV': WAV_AUDIO,
    'AMR': AMR_AUDIO,
    'AVI': AVI_VIDEO,
    'MP4': MP4_VIDEO,
    'MOV': MOV_VIDEO,
    'MKV': MKV_VIDEO,
    'DOC': DOC_DOCUMENTS,
    'DOCX': DOCX_DOCUMENTS,
    'TXT': TXT_DOCUMENTS,
    'PDF': PDF_DOCUMENTS,
    'XLSX': XLSX_DOCUMENTS,
    'PPTX': PPTX_DOCUMENTS,
    'ZIP': ZIP_ARCHIVES,
    'GZ': GZ_ARCHIVES,
    'TAR': TAR_ARCHIVES,
    'RAR': RAR_ARCHIVES
}

FOLDERS = []
EXTENSION = set()
UNKNOWN = set()

def get_extension(filename: str) -> str:
    return Path(filename).suffix[1:].upper()  # перетворюємо розширення файлу на назву папки jpg -> JPG

def scan(folder: Path) -> None:
    for item in folder.iterdir():
        # Якщо це папка то додаємо її до списку FOLDERS і переходимо до наступного елемента папки
        if item.is_dir():
            # перевіряємо, щоб папка не була тією в яку ми складаємо вже файли
            if item.name not in ('archives', 'video', 'audio', 'documents', 'images', 'MY_OTHER'):
                FOLDERS.append(item)
                # скануємо вкладену папку
                scan(item)  # рекурсія
            continue  # переходимо до наступного елементу в сканованій папці

        #  Робота з файлом
        ext = get_extension(item.name)  # беремо розширення файлу
        fullname = folder / item.name  # беремо шлях до файлу
        if not ext:  # якщо файл немає розширення то додаєм до невідомих
            MY_OTHER.append(fullname)
        else:
            try:
                container = REGISTER_EXTENSION[ext]
                EXTENSION.add(ext)
                container.append(fullname)
            except KeyError:
                # Якщо ми не зареєстрували розширення у REGISTER_EXTENSION, то додаємо до невідомих
                UNKNOWN.add(ext)
                MY_OTHER.append(fullname)


def handle_media(filename: Path, target_folder: Path) -> None:
    target_folder.mkdir(exist_ok=True, parents=True)
    filename.replace(target_folder / normalize(filename.name))

def handle_other(filename: Path, target_folder: Path) -> None:
    target_folder.mkdir(exist_ok=True, parents=True)
    filename.replace(target_folder / normalize(filename.name))

def handle_archive(filename: Path, target_folder: Path) -> None:
    target_folder.mkdir(exist_ok=True, parents=True)  # робимо папку для архіва
    folder_for_file = target_folder / normalize(filename.name.replace(filename.suffix, ''))
    folder_for_file.mkdir(exist_ok=True, parents=True)
    try:
        shutil.unpack_archive(filename, folder_for_file)  # TODO: Check!
    except shutil.ReadError:
        print('It is not archive')
        folder_for_file.rmdir()
    filename.unlink()

def handle_folder(folder: Path):
    try:
        folder.rmdir()
    except OSError:
        print(f"Can't delete folder: {folder}")

def main(folder: Path):
    scan(folder)
    for file in JPEG_IMAGES:
        handle_media(file, folder / 'images' / 'JPEG')
    for file in JPG_IMAGES:
        handle_media(file, folder / 'images' / 'JPG')
    for file in PNG_IMAGES:
        handle_media(file, folder / 'images' / 'PNG')
    for file in SVG_IMAGES:
        handle_media(file, folder / 'images' / 'SVG')

    for file in MP3_AUDIO:
        handle_media(file, folder / 'audio' / 'MP3')
    for file in OGG_AUDIO:
        handle_media(file, folder / 'audio' / 'OGG')
    for file in WAV_AUDIO:
        handle_media(file, folder / 'audio' / 'WAV')
    for file in AMR_AUDIO:
        handle_media(file, folder / 'audio' / 'AMR')

    for file in AVI_VIDEO:
        handle_media(file, folder / 'video' / 'AVI')
    for file in MP4_VIDEO:
        handle_media(file, folder / 'video' / 'MP4')
    for file in MOV_VIDEO:
        handle_media(file, folder / 'video' / 'MOV')
    for file in MKV_VIDEO:
        handle_media(file, folder / 'video' / 'MKV')

    for file in DOC_DOCUMENTS:
        handle_media(file, folder / 'documents' / 'DOC')
    for file in DOCX_DOCUMENTS:
        handle_media(file, folder / 'documents' / 'DOCX')
    for file in TXT_DOCUMENTS:
        handle_media(file, folder / 'documents' / 'TXT')
    for file in PDF_DOCUMENTS:
        handle_media(file, folder / 'documents' / 'PDF')
    for file in XLSX_DOCUMENTS:
        handle_media(file, folder / 'documents' / 'XLSX')
    for file in PPTX_DOCUMENTS:
        handle_media(file, folder / 'documents' / 'PPTX')

    

    for file in MY_OTHER:
        handle_other(file, folder / 'MY_OTHER')
    
    for file in ZIP_ARCHIVES:
        handle_archive(file, folder / 'ARCHIVES' / 'ZIP')
    for file in GZ_ARCHIVES:
        handle_archive(file, folder / 'ARCHIVES' / 'GZ')
    for file in TAR_ARCHIVES:
        handle_archive(file, folder / 'ARCHIVES' / 'TAR')
    for file in RAR_ARCHIVES:
        handle_archive(file, folder / 'ARCHIVES' / 'RAR')

    

    for folder in FOLDERS[::-1]:
        handle_folder(folder)

if __name__ == "__main__":
    if sys.argv[1]:
        folder_for_scan = Path(sys.argv[1])
        print(f'Start in folder: {folder_for_scan.resolve()}')
        main(folder_for_scan.resolve())

# Oleksandr Kuspys


from prompt_toolkit.lexers import Lexer
from prompt_toolkit.styles.named_colors import NAMED_COLORS
from prompt_toolkit.completion import NestedCompleter


class RainbowLexer(Lexer):
    def lex_document(self, document):
        colors = list(sorted({"Teal": "#008080"}, key=NAMED_COLORS.get))

        def get_line(lineno):
            return [
                (colors[i % len(colors)], c)
                for i, c in enumerate(document.lines[lineno])
            ]

        return get_line


Completer = NestedCompleter.from_nested_dict(
    {
        "hello": None,
        "help": None,
        "exit": None,
        "close": None,
        ".": None,
        "edit": None,
        "phone": None,
        "save": None,
        "show": None,
        "all": None,
        "show all": None,
        "add": None,
        "add contact": None,
        "add phone": None,
        "add email": None,
        "add birthday": None,
        "change": None,
        "change phone": None,
        "change email": None,
        "change birthday": None,
        "delete contact": None,
        "delete phone": None,
        "delete email": None,
        "delete birthday": None,
        "find name": None,
        "get birthday": None,
        "add note": None,
        "find note": None,
        "find by tag": None,
        "show all notes": None,
        "change note": None,
        "change text": None,
        "change tag": None,
        "delete note": None,
        "add tag": None,
        "close": None,
    }
)



from collections import UserDict
from datetime import datetime
from .prompt_tool import Completer, RainbowLexer
from prompt_toolkit import prompt
import pickle
import re


class PhoneException(Exception):
    pass


class EmailException(Exception):
    pass


class BDException(Exception):
    pass


class Field:
    def __init__(self, value) -> None:
        self.__private_value = None
        self.value = value

    @property
    def value(self):
        return self.__private_value

    @value.setter
    def value(self, value):
        self.__private_value = value


class Name(Field):
    pass


class Address(Field):
    pass


class Phone(Field):
    def __init__(self, value) -> None:
        super().__init__(value)

    @Field.value.setter
    def value(self, value: str):
        value = value.strip()
        # перевірка довжини номера та чи всі введені значення є  цифрами
        right_len = len(value) == 10 or len(value) == 12
        if value.isdigit() and right_len:
            Field.value.fset(self, value)
        else:
            raise PhoneException


class Email(Field):
    def __init__(self, value) -> None:
        super().__init__(value)

    @Field.value.setter
    def value(self, value: str):
        result = re.findall(r"[a-zA-z]{1}[\w\.]+@[a-zA-z]+\.[a-zA-z]{2,}", value)
        if result:
            Field.value.fset(self, value)
        else:
            raise EmailException


class Birthday(Field):
    def __init__(self, value) -> None:
        super().__init__(value)

    @Field.value.setter
    def value(self, value: str):
        value.strip()
        # перевірка чи вік є в діавзоні 0 - 150 р
        delta = datetime.now() - datetime.strptime(value, "%d.%m.%Y")
        if (delta.days < 0) or (delta.days > 53000):
            raise BDException
        else:
            Field.value.fset(self, value)


class Record:
    def __init__(
        self,
        name: Name,
        addres: Address = None,
        phone: Phone = None,
        email: Email = None,
        birthday: Birthday = None,
    ):
        self.name = name
        self.addres = addres
        self.phones = list()
        if phone:
            self.phones.append(phone.value)

        self.email = email
        self.birthday = birthday

    def add_email(self, email: Email):
        self.email = email

    def add_bd(self, birthday: Birthday):
        self.birthday = birthday

    def add_phone(self, phone: Phone):
        self.phones.append(phone.value)

    def change_phone(self, old_phone, new_phone):
        index = self.phones.index(old_phone)
        self.phones[index] = new_phone

    def is_birthday_next_days(self, n: int) -> bool:
        if self.birthday.value:
            birthday_date = datetime.strptime(self.birthday.value, "%d.%m.%Y")
            current_date = datetime.now()
            birthday_date = birthday_date.replace(year=current_date.year)
            if current_date > birthday_date:
                birthday_date = birthday_date.replace(year=current_date.year + 1)

            delta_days = birthday_date - current_date

            if 0 <= delta_days.days < n:
                return True
            return False

    def __repr__(self) -> str:
        if self.birthday == None:
            bd = "_"
        else:
            bd = self.birthday.value

        if self.email == None:
            em = "_"
        else:
            em = self.email.value

        if self.phones == list():
            ph = "_"
        else:
            ph = str(self.phones)

        return "|{:^20}|{:^20}|{:^20}|".format(ph, bd, em)


class AddressBook(UserDict):
    def add_record(self, record: Record):
        self.data[record.name.value] = record
        return self.data

    def save_ab(self):
        with open("addressbook.pkl", "wb") as file:
            pickle.dump(self.data, file)

    def open_ab(self):
        with open("addressbook.pkl", "rb") as file:
            self.data = pickle.load(file)


# функція для обробки винятків
def input_error(func):
    def inner(*args):
        try:
            return func(*args)
        except IndexError:
            return "Будь ласка, введіть всю необхідну інформацію!"
        except KeyError:
            return "Даний контакт відсутній у книзі контактів!"
        except PhoneException:
            return "Не правильний формат номеру телефона. Очікується - 0961010100 або 380961010100"

        except EmailException:
            return "Не правильний формат електронної пошти. Очікується - test@test.com"
        except BDException:
            return "Дата народження вказано не правильно, вік виходить за межі діапазону 0-150"

        except ValueError:
            return "Не правильний формат дати народження. Повинен бути дд.мм.рррр"

    return inner


AB = AddressBook()
try:
    AB.open_ab()
except FileNotFoundError:
    AB = AddressBook()


@input_error
def add_handler(user_info: str):
    name = user_info[0].lstrip()
    rec = Record(Name(name))
    AB.add_record(rec)
    return f"Користувач{user_info[0]} доданий"


@input_error
def add_number(user_input: str):
    name = user_input[0].lstrip()
    phone = user_input[1]
    rec = AB[name]
    rec.add_phone(Phone(phone))
    return f"Номер {phone} додано до контакту {name}"


@input_error
def add_email(user_input: str):
    name = user_input[0].lstrip()
    email = user_input[1]
    rec = AB[name]
    rec.add_email(Email(email))
    return f"Електронна пошта {email} додано до контакту {name}"


@input_error
def add_birthday(user_input: str):
    name = user_input[0].lstrip()
    bd = user_input[1].lstrip()
    rec = AB[name]
    rec.add_bd(Birthday(bd))
    return f"Дата народження {bd} додано до контакту {name}"


def help_handler(*args):
    print("add contact, name - Додати новий контакт до Книги контактів")
    print("add phone, name, phone - Додати номер телефону до існуючого контакту ")
    print("add email, name, email - Додати електронну пошту до існуючого контакту")
    print(
        "add birthday, name, birthday	- Додати  день народження до існуючого контакту"
    )
    print(
        "change phone, name, old phone, new phone - Змінити номер телефону вказаного контакту"
    )
    print(
        "change birthday, name, birthday - Змінити день народження вказаного контакту"
    )
    print(
        "delete contact, name - 	Видалити контакт за вказаним ім’ям з Книги контактів"
    )
    print(
        "delete phone, name, phone - Видалити номер телефону контакту з Книги контактів"
    )
    print(
        "delete email, name, email - Видалити електронну пошту контакту з Книги контактів"
    )
    print(
        "delete birthday, name, birthday - Видалити день народження контакту з Книги контактів"
    )
    print(
        "get birthday, days - Показати в кого день народження через введену кількість днів"
    )
    print("show all - Показати всі контакти з Книги контактів")
    print("find name, name - Знайти контакт")
    print(
        "exit, close, goodbye	(одна команда на вибір)- Закінчити роботу або повернутися в головне меню"
    )

    return " "


@input_error
def ch_phone(user_input: str):
    name = user_input[0].lstrip()
    old_phone = Phone(user_input[1].lstrip())
    new_phone = Phone(user_input[2].lstrip())
    rec = AB[name]
    if old_phone.value in rec.phones:
        rec.change_phone(old_phone.value, new_phone.value)
        return f"Номер телефону користувача {name} змінено на {new_phone.value}"
    return f"Номер телефону {old_phone.value} не знайдено!"


@input_error
def change_email(user_input: str):
    name = user_input[0].lstrip()
    email = user_input[1]
    rec = AB[name]
    rec.add_email(Email(email))
    return f"Електронну пошту контакту {name} змінено на {email} "


@input_error
def change_birthday(user_input: str):
    name = user_input[0].lstrip()
    bd = user_input[1].lstrip()
    rec = AB[name]
    rec.add_bd(Birthday(bd))
    return f"День народження контакту {name} змінено на {bd} "


@input_error
def del_contact(user_input: str):
    name = user_input[0].lstrip()
    AB.pop(name)
    return f"Контакт {name} видалено"


def show_all(*args):
    print("|{:^20}|{:^20}|{:^20}|{:^20}|".format("Name", "Phone", "Birthday", "Email"))
    for k, v in AB.items():
        print("|{:^19}".format(k), v)
    return ""


@input_error
def del_phone(user_input: str):
    name = user_input[0].lstrip()
    phone = Phone(user_input[1].lstrip())
    rec = AB[name]
    if phone.value in rec.phones:
        rec.phones.remove(phone.value)
        # print(rec.phones)
        return f"Номер телефону {phone.value} користувача {name} видалено"
    return f"Номер телефону {phone.value} не знайдено!"


@input_error
def del_email(user_input: str):
    name = user_input[0].lstrip()
    rec = AB[name]
    rec.add_email(None)
    return f"Електронну пошту контакту {name} видалено "


@input_error
def del_birthday(user_input: str):
    name = user_input[0].lstrip()
    rec = AB[name]
    rec.add_bd(None)
    return f" Для контакту {name} дату народження видалено"


@input_error
def save_adbook(*args):
    AB.save_ab()
    return f"Книга контактів збережена"


@input_error
def find_name(user_input: str):
    name = user_input[0].lstrip()
    for key in AB.keys():
        if name == key:
            record = AB[name]
            return f"{name}: {record}"
    return f"Контакт {name} відсутній у книзі контактів"


def get_birthday(user_input: str):
    try:
        n = int(user_input[0])
    except ValueError:
        return "введіть число в діапазоні 1 - 354 "
    except IndexError:
        return "Будь ласка, введіть всю необхідну інформацію!"

    result = list()

    if n in range(1, 365):
        for name, record in AB.items():
            if record.birthday:
                if record.is_birthday_next_days(n):
                    result.append(name)
    else:
        return f"введіть число в діапазоні 1 - 364 "

    if result:
        return f"Через {n} днів день народження святкують {result}"
    return f"Через {n} днів день народження не святкує жоден з контактів"


COMMANDS = {
    add_handler: "add contact",
    add_number: "add phone",
    add_email: "add email",
    add_birthday: "add birthday",
    help_handler: "help",
    ch_phone: "change phone",
    change_email: "change email",
    change_birthday: "change birthday",
    del_contact: "delete contact",
    show_all: "show all",
    save_adbook: "save",
    del_phone: "delete phone",
    del_email: "delete email",
    del_birthday: "delete birthday",
    find_name: "find name",
    get_birthday: "get birthday",
}


def command_parser(user_input: str):
    user_info = user_input.split(",")
    command = user_info[0]

    for key, value in COMMANDS.items():
        if command.lower() == value:
            return key(user_info[1:])
    return "Не відома команда. Спробуйте ще раз! Команду та основні блоки розділіть комами!"


def main():
    print('Вас вітає книга контактів! Введіть команду чи "help" для допомоги')
    while True:
        user_input = prompt(
            ">>> ",
            completer=Completer,
            lexer=RainbowLexer(),
        ).strip()
        # user_input = input(">>> ")
        if user_input.lower() in ["good bye", "exit", "close"]:
            AB.save_ab()
            print("Good bye!")
            break

        result = command_parser(user_input)

        print(result)


if __name__ == "__main__":
    main()

# Ivan Markovskyi



from .calculator import calc
from pathlib import Path
from .prompt_tool import Completer, RainbowLexer
from prompt_toolkit import prompt
from . import adbook, note_book, sort_folder


def run_folder():
    folder_path = input("Введіть шлях до теки для сортування: ")
    sort_folder.main(Path(folder_path))
    return "Сортування завершено успішно"


bot_command_dict = {
    "1": adbook.main,
    "2": note_book.run_notebook,
    "3": run_folder,
    "4": calc,
}


def assistant_bot():
    print("Вас вітає персональний помічник FriendlyHandbook")
    print(
        """
    Виберіть одну з наступних опцій:
    - Книга контактів (PhoneBook) -> Натисніть '1'
    - Нотатки (NoteBook) -> Натисніть '2'
    - Сортувач папок (CleanFolder) -> Натисніть '3'
    - Калькулятор (Calculator) -> Натисніть '4'
    - Вийти з помічника -> Натисніть '0'
    """
    )

    while True:
        command = prompt(
            "Введіть номер опції (від 0 до 4): ",
            completer=Completer,
            lexer=RainbowLexer(),
        ).strip()

        if command == "0":
            raise SystemExit("\nДо побачення!\n")

        elif command in bot_command_dict.keys():
            handler = bot_command_dict[command]
            answer = handler()
            print(answer)

        else:
            print("Некоректне число. Будь ласка, введіть число від 0 до 4")


if __name__ == "__main__":
    assistant_bot()


# Oleksandr Yukha



import os
import pickle
from collections import UserDict
from .prompt_tool import Completer, RainbowLexer
from prompt_toolkit import prompt


SAVE_FILENAME = "notebook.pkl"  # Ім'я файлу для запису данних


class Field_NOTEBOOK:
    def __init__(self, value):
        self.value = value


class Tag(Field_NOTEBOOK):
    def __eq__(self, other):
        return isinstance(other, Tag) and self.value == other.value


class Name(Field_NOTEBOOK):
    pass


class Record:
    def __init__(self, name: Name, tags: list, text: str):
        self.name = name
        self.tags = [Tag(tag) for tag in tags]
        self.text = text

    def add_tag(self, tag):
        hashtag = Tag(tag)
        if hashtag not in self.tags:
            self.tags.append(hashtag)

    def find_tag(self, value):
        for tag in self.tags:
            if tag.value == value:
                return tag
        return None

    def delete_tag(self, tag):
        if tag in self.tags:
            self.tags.remove(tag)

    def edit_text(self, new_text):
        self.text = new_text


class Notebook(UserDict):
    def add_record(self, record: Record):
        self.data[record.name.value] = record

    def save_to_file(self):  # збереження файлу
        with open(SAVE_FILENAME, "wb") as file:
            pickle.dump(self.data, file)

    def load_from_file(self):  # завантаження файлу
        if os.path.exists(SAVE_FILENAME):
            try:
                with open(SAVE_FILENAME, "rb") as file:
                    self.data = pickle.load(file)
            except FileNotFoundError:
                print(f"Файл '{SAVE_FILENAME}' не знайдений.")
        else:
            # Не видаляти дані, якщо файл не існує
            print(f"Файл '{SAVE_FILENAME}' не існує. Дані не завантажено.")

    def find_record(self, value):
        return self.data.get(value)

    def edit_record(self, name, new_tags, new_text):
        if name in self.data:
            record = self.data[name]
            record.tags = [Tag(tag) for tag in new_tags]
            record.text = new_text
            self.data[name] = record
            print(f"Нотатка {name} була змінена.")
        else:
            print(f"Нотатка '{name}' не знайдена.")

    def search_by_tag(self, tag):
        matching_records = []
        for record in self.data.values():
            if any(tag == t.value for t in record.tags):
                matching_records.append(record)
        return matching_records


def run_notebook():
    note_book = Notebook()
    note_book.load_from_file()

    while True:
        print("\nOptions:")
        print("1. Додати нотатку           (add note)")
        print("2. Знайти нотатку за назвою (find note)")
        print("3. Пошук за тегами          (find by tag)")
        print("4. Вивести всі нотатки      (show all notes)")
        print("5. Редагувати нотатку       (change note)")
        print("6. Редагувати текст нотатки (change text)")
        print("7. Редагувати теги нотатки  (change tag)")
        print("8. Видалити нотатку         (delete note)")
        print("9. Додати теги до нотатки   (add tag)")
        print("0. Вихід з записника        (close)")

        # choice = input("Виберіть опцію: ")
        choice = prompt(
            "Виберіть опцію: ",
            completer=Completer,
            lexer=RainbowLexer(),
        ).strip()

        if choice == "1" or choice == "add note":
            name = input("Вкажіть ім'я нотатки: ")
            hashtags = input(
                "Вкажіть теги нотатки (Якщо кілька, то вказати через кому): "
            ).split(", ")
            text = input("Введіть текст нотатки: ")

            name_field = Name(name.strip())
            record = Record(name_field, hashtags, text)
            note_book.add_record(record)
            print(f"Нотатка {name} добавлена до записника.")
            pass

        elif choice == "2" or choice == "find note":
            search_term = input("Вкажіть ім'я нотатки для пошук: ")
            record = note_book.find_record(search_term)
            if record:
                print(f"Name: {record.name.value}")
                print("Tags:")
                for tag in record.tags:
                    print(tag.value)
                print("Text:")
                print(record.text)
            else:
                print(f"Нотатку з ім'ям '{search_term}' не було знайдено.")
            pass

        elif choice == "3" or choice == "find by tag":
            tag_to_search = input("Введіть тег для пошуку: ")
            matching_records = note_book.search_by_tag(tag_to_search)
            if matching_records:
                print("Matching Records:")
                for record in matching_records:
                    print(f"Name: {record.name.value}")
                    print("Tags:")
                    for tag in record.tags:
                        print(tag.value)
                    print("Text:")
                    print(record.text)
            else:
                print(f"Нотаток з тегом '{tag_to_search}' не знайдено.")
            pass

        elif choice == "4" or choice == "show all notes":  # Виведення всіх нотаток
            print("Список всіх нотаток:")
            for name, record in note_book.data.items():
                print(f"Name: {record.name.value}")
                for tag in record.tags:
                    print(f"Tags:{tag.value}")
                print(f"Text: {record.text}")

            pass

        elif choice == "5" or choice == "change note":
            edit_name = input("Вкажіть ім'я нотатки для редагування: ")
            new_tags = input(
                "Вкажіть нові теги (Якщо кілька, то вказати через кому): "
            ).split(", ")
            new_text = input("Введіть новий текст: ")
            note_book.edit_record(edit_name.strip(), new_tags, new_text)
            pass

        elif choice == "6" or choice == "change text":
            edit_name = input("Вкажіть ім'я нотатки для редагування тексту: ")
            record = note_book.find_record(edit_name.strip())
            if record:
                new_text = input("Введіть новий текст: ")
                record.edit_text(new_text)
                print(f"Текст нотатки {edit_name} був змінений.")
            else:
                print(f"Нотатку з ім'ям '{edit_name}' не було знайдено.")

        elif choice == "7" or choice == "change tag":
            edit_tags_name = input("Вкажіть ім'я нотатки для редагування тегів: ")
            record = note_book.find_record(edit_tags_name.strip())
            if record:
                new_tags = input(
                    "Вкажіть нові теги для нотатки (Якщо кілька, то вказати через кому): "
                ).split(", ")
                record.tags = [Tag(tag.strip()) for tag in new_tags]
                print(f"Теги для нотатки {edit_tags_name} були змінені.")
            else:
                print(f"Нотатку з ім'ям '{edit_tags_name}' не було знайдено.")

        elif choice == "8" or choice == "delete note":
            delete_term = input("Вкажіть ім'я нотатки для видалення: ")
            record = note_book.find_record(delete_term)
            if record:
                del note_book.data[delete_term]
                print(f"Нотатка {delete_term} була видалена з записника.")
            else:
                print(f"Нотатку з ім'ям '{delete_term}' не було знайдено.")
            pass

        elif choice == "9" or choice == "add tag":
            edit_tags_name = input(
                "Вкажіть ім'я нотатки, до якої потрібно додати теги: "
            )
            record = note_book.find_record(edit_tags_name)
            if record:
                new_tags = input(
                    "Вкажіть нові теги для нотатки (Якщо кілька, то вказати через кому): "
                ).split(", ")
                for tag in new_tags:
                    record.add_tag(tag.strip())
                print(f"Теги для нотатки {edit_tags_name} були додані.")
            else:
                print(f"Нотатку з ім'ям '{edit_tags_name}' не було знайдено.")
            pass

        elif choice == "0" or choice == "close":
            # Зберегти дані перед виходом з програми
            note_book.save_to_file()
            print(f"Дані збережено в файлі '{SAVE_FILENAME}'")
            print("До побачення!")
            from main import assistant_bot

            assistant_bot()
            break

        else:
            print("Не коректний вибір. Виберіть опцію.")

        # Завершуємо пункт меню і чекаємо Enter перед поверненням до головного меню.
        input("Для продовження, натисніть Enter: ")


if __name__ == "__main__":
    run_notebook()

# Kovaleno Oleksandr



# Функція для додавання
def add(x, y):
    return x + y


# Функція для віднімання
def sub(x, y):
    return x - y


# Функція для множення
def multiply(x, y):
    return x * y


# Функція для ділення
def div(x, y):
    if y == 0:
        return "На нуль ділити не можна"
    return x / y


# Основний цикл програми
def calc():
    while True:

        choice = input("Введіть операцію ('+', '-', '*', '/',     Для виходу -> 'q'): ")

        if choice == 'q':
            print("Програма завершена.")
            break

        num1 = float(input("Введіть перше число: "))
        num2 = float(input("Введіть друге число: "))

        if choice == '+':
            print("Результат:", add(num1, num2))
        elif choice == '-':
            print("Результат:", sub(num1, num2))
        elif choice == '*':
            print("Результат:", multiply(num1, num2))
        elif choice == '/':
            print("Результат:", div(num1, num2))
        else:
            print("Невірний ввід. Спробуйте ще раз.")
    return ('Повернення в попереднє Меню')

if __name__ == "__main__":
    calc()



from setuptools import setup, find_namespace_packages

setup(
    name="FriendlyHandbook",
    version="0.1.2",
    description="«FriendlyHandbook» - це корисна програма з інтерфейсом командного рядка, яка містить контактну книгу, нотатки, калькулятор та може аналізувати папки.",
    url="https://github.com/filinmbg/FriendlyHandbook",
    author="JustPython",
    author_email="filinmbg@gmail.com",
    license="MIT",
    packages=find_namespace_packages(),
    install_requires=["prompt-toolkit==3.0.39"],
    entry_points={
        "console_scripts": ["helloworld = FriendlyHandbook.main:assistant_bot"]
    },
)
